@@ -535,7 +535,7 @@ HRESULT get_node_text(HTMLDOMNode *node, BSTR *ret)
     if (!wstrbuf_init(&buf))
         return E_OUTOFMEMORY;
     wstrbuf_append_node_rec(&buf, node->nsnode);
-    if(buf.buf) {
+    if(buf.buf && *buf.buf) {
         *ret = SysAllocString(buf.buf);
         if(!*ret)
             hres = E_OUTOFMEMORY;
@@ -1207,7 +1207,7 @@ static HRESULT WINAPI HTMLTxtRange_move(IHTMLTxtRange *iface, BSTR Unit,
 
     if(!Count) {
         *ActualCount = 0;
-        return IHTMLTxtRange_collapse(&This->IHTMLTxtRange_iface, TRUE);
+        return IHTMLTxtRange_collapse(&This->IHTMLTxtRange_iface, VARIANT_TRUE);
     }
 
     switch(unit) {
@@ -1219,7 +1219,7 @@ static HRESULT WINAPI HTMLTxtRange_move(IHTMLTxtRange *iface, BSTR Unit,
         *ActualCount = move_by_chars(&start, Count);
 
         set_start_point(This, &start);
-        IHTMLTxtRange_collapse(&This->IHTMLTxtRange_iface, TRUE);
+        IHTMLTxtRange_collapse(&This->IHTMLTxtRange_iface, VARIANT_TRUE);
         free_rangepoint(&start);
         break;
     }
@@ -1232,7 +1232,7 @@ static HRESULT WINAPI HTMLTxtRange_move(IHTMLTxtRange *iface, BSTR Unit,
         *ActualCount = move_by_words(&start, Count);
 
         set_start_point(This, &start);
-        IHTMLTxtRange_collapse(&This->IHTMLTxtRange_iface, TRUE);
+        IHTMLTxtRange_collapse(&This->IHTMLTxtRange_iface, VARIANT_TRUE);
         free_rangepoint(&start);
         break;
     }
