@@ -50,17 +50,40 @@ static inline WS_XML_NODE_TYPE node_type( const struct node *node )
     return node->hdr.node.nodeType;
 }
 
+struct prop_desc
+{
+    ULONG size;
+    BOOL  readonly;
+    BOOL  writeonly;
+};
+
+struct prop
+{
+    void  *value;
+    ULONG  size;
+    BOOL   readonly;
+    BOOL   writeonly;
+};
+
+ULONG prop_size( const struct prop_desc *, ULONG ) DECLSPEC_HIDDEN;
+void prop_init( const struct prop_desc *, ULONG, struct prop *, void * ) DECLSPEC_HIDDEN;
+HRESULT prop_set( const struct prop *, ULONG, ULONG, const void *, ULONG ) DECLSPEC_HIDDEN;
+HRESULT prop_get( const struct prop *, ULONG, ULONG, void *, ULONG ) DECLSPEC_HIDDEN;
+
 struct channel
 {
     WS_CHANNEL_TYPE         type;
     WS_CHANNEL_BINDING      binding;
+    WS_CHANNEL_STATE        state;
     ULONG                   prop_count;
-    WS_CHANNEL_PROPERTY     prop[9];
+    struct prop             prop[9];
 };
 
 HRESULT create_channel( WS_CHANNEL_TYPE, WS_CHANNEL_BINDING, const WS_CHANNEL_PROPERTY *,
                         ULONG, struct channel ** ) DECLSPEC_HIDDEN;
 void free_channel( struct channel * ) DECLSPEC_HIDDEN;
+HRESULT open_channel( struct channel *, const WS_ENDPOINT_ADDRESS * ) DECLSPEC_HIDDEN;
+HRESULT close_channel( struct channel * ) DECLSPEC_HIDDEN;
 
 static inline void *heap_alloc( SIZE_T size )
 {
